unit unLiveInCode;
{
    LiveInCode - a graphical front-end to LineInCode
    Copyright (C) 2003-2004 Roman Mamedov
    <E-Mail: roman@rm.pp.ru, Website: http://rm.pp.ru/?1.liveincode>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, StrUtils, unLincoTrd, unrm_free_utils, unrm_filefuncs, unRM_Messages,
  Menus, ComCtrls, ExtCtrls, shellapi, unRM_IniWork, unRM_AboutBox, XPMan;

type
  TfmMain = class(TForm)
    btRecord: TButton;
    btStop: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Help1: TMenuItem;
    Exit1: TMenuItem;
    About1: TMenuItem;
    pFilename: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    pStatus: TPanel;
    pDuration: TPanel;
    Label3: TLabel;
    tmr: TTimer;
    pFilesize: TPanel;
    Label4: TLabel;
    btPause: TButton;
    Configure1: TMenuItem;
    N1: TMenuItem;
    pBitrate: TPanel;
    Label5: TLabel;
    XPManifest1: TXPManifest;
    ools1: TMenuItem;
    RecordingMixer1: TMenuItem;
    nProfiles: TMenuItem;
    VolumeControl1: TMenuItem;
    procedure btRecordClick(Sender: TObject);
    procedure btStopClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure tmrTimer(Sender: TObject);
    procedure pFilenameClick(Sender: TObject);
    procedure btPauseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure About1Click(Sender: TObject);
    procedure Configure1Click(Sender: TObject);
    procedure RecordingMixer1Click(Sender: TObject);
    procedure VolumeControl1Click(Sender: TObject);
  private
    procedure ProfileItemClick(Sender: TObject);
    { Private declarations }
  public
    { Public declarations }
    procedure FileLabelAsLink(state:Boolean);
    procedure UpdateProfileList;
  end;

var
  fmMain: TfmMain;
  LincoTrd: TLincoTrd;
  nowRecording:Boolean;
  RecStartTime, PauseStartTime, PauseTotalTime:TDateTime;
  RecFileName, CmdShell, EncoderOgg:String;
  ini:TRMIniFile;

function SetLincoStopFlag(state:Boolean):Boolean;  
function SetLincoPauseFlag(state:Boolean):Boolean;

implementation

uses unCommonTypes, unConfigure;

{$R *.DFM}

procedure TfmMain.ProfileItemClick(Sender: TObject);
var item:TMenuItem;
begin
  item := (Sender as TMenuItem);
  item.Checked := True;
  ini.Write(INIGENERAL, 'profile', item.Caption);
end;

procedure TfmMain.UpdateProfileList;
var filenames:TStringList;
    item:TMenuItem;
    c:Integer;
    FocusOn:String;
begin
  nProfiles.Clear;
  filenames := EnumFiles(PROFILEDIR + '\*.ini',False);
  if filenames.Count > 0 then begin
    for c:=0 to filenames.Count - 1 do begin
      filenames.Strings[c] := ChangeFileExt(filenames.Strings[c],'');
      item :=TMenuItem.Create(Self);
      item.Caption := filenames.Strings[c];
      item.OnClick := fmMain.ProfileItemClick;
      item.RadioItem := True;
      nProfiles.Add(item);
    end;
  end else begin
    item :=TMenuItem.Create(Self);
    item.Caption := 'Default';
    item.OnClick := fmMain.ProfileItemClick;
    item.RadioItem := True;    
    nProfiles.Add(item);
  end;
  nProfiles.Items[0].Checked := True;
  FocusOn := ini.Read(INIGENERAL,'profile','Default');
  for c:=0 to nProfiles.Count - 1 do begin
    if nProfiles.Items[c].Caption = FocusOn then begin
      nProfiles.Items[c].Checked := True;
      Break;
    end;
  end;
end;

procedure TfmMain.FileLabelAsLink(state:Boolean);
begin
  if state then begin
    pFilename.Font.Style := [fsUnderline];
    pFilename.Font.Color := clBlue;
    pFilename.Cursor := crHandPoint;
  end else begin
    pFilename.Font.Style := [];
    pFilename.Font.Color := clWindowText;
    pFilename.Cursor := crDefault;
  end;
end;

function SetLincoFlag(const name:string;state:Boolean):Boolean;
var filename:string;
begin
  filename := 'linco.'+name+'.flag';
  if state then begin
    Result := MakeEmptyFile(filename);
  end else begin
    Result := DeleteFile(filename);
  end;
end;

function SetLincoPauseFlag(state:Boolean):Boolean;
begin
  Result := SetLincoFlag('pause',state);
end;

function SetLincoStopFlag(state:Boolean):Boolean;
begin
  Result := SetLincoFlag('stop',state);
end;

function DateTimeToFileName:String;
begin
  Result := FormatDateTime('yyyy-mm-dd" at "hh-nn-ss',Now);
end;

procedure TfmMain.btRecordClick(Sender: TObject);
var lincmd, custcmd, extn, dir, cmd,
    rate, bits, channels, oggquality,
    EncoderOgg, profilename: string;
    pini:TRMIniFile;
begin
  //Get settings from the ini-file
  profilename := ini.Read(INIGENERAL, 'profile', 'Default');
  pini := TRMIniFile.Create(PROFILEDIR + '\' + profilename + '.ini'); 
  dir := pini.Read(INIGENERAL, 'dir', '');
  if dir <> '' then
    dir := IncludeTrailingPathDelimiter(dir);
  bits := pini.Read(INIGENERAL, 'bits', '16');
  channels := pini.Read(INIGENERAL, 'channels', '2');
  rate := pini.Read(INIGENERAL, 'rate', '44100');
  oggquality := pini.Read(INIOGG, 'quality', '6');
  EncoderOgg := pini.Read(INIOGG, 'encoder', 'oggenc');

  //construct the first part of the command-line
  lincmd := CmdShell + ' /C linco --flag-files -B ' + bits + ' -C ' + channels + ' -R ' + rate + ' | ';

  //find out what kind of the second part to construct
  if pini.Read(INIOGG, 'enabled', True) then begin
    //Ogg command-line:
    RecFileName := dir + DateTimeToFileName + '.ogg';
    cmd := lincmd + EncoderOgg + ' - -q '+ oggquality
    + ' -r -B ' + bits
    + ' -C ' + channels
    + ' -R ' + rate
    + ' -o "' + RecFileName + '"';
  end else begin
    //Custom command-line:
    custcmd := pini.Read(INICUSTOM, 'commandline', DEFAULTCUSTOMLINE);
    //To show the file size and link we need to know full filename, with extension.
    //Get 8 chars after %f, the file extension(anyone uses 8-chars long extensions? :)
    extn := Copy(custcmd, Pos('%f', custcmd) + 2, 8);
    //Get only the part before a space or a quote(the extension itself, not any junk after it).
    if Pos(' ', extn) > 0 then
      extn := Copy(extn, 0, Pos(' ', extn) - 1);
    if Pos('"', extn) > 0 then
      extn := Copy(extn, 0, Pos('"', extn) - 1);
    RecFileName := dir + DateTimeToFileName + extn;
    custcmd := AnsiReplaceStr(custcmd, '%f', dir + DateTimeToFileName);
    cmd := lincmd + custcmd;
  end;
  pini.Free;
  pFilename.Caption := ExtractFileName(RecFileName);
  pFilename.Hint := RecFileName;
  LincoTrd := TLincoTrd.Create(True);
  LincoTrd.FreeOnTerminate := True;
  LincoTrd.CommandLine := cmd;
  RecStartTime := Now;
  PauseTotalTime := 0;
  LincoTrd.Resume;
end;

procedure TfmMain.btStopClick(Sender: TObject);
begin
  pStatus.Caption := 'Stopping...';
  SetLincoStopFlag(True);
end;

procedure TfmMain.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TfmMain.tmrTimer(Sender: TObject);
var filesize:Integer;
    sFilesize:String;
    duration:TDateTime;
const
  KB = 1024;
  MB = 1024*KB;
begin
  duration := Now - RecStartTime - PauseTotalTime;
  pDuration.Caption := FormatDateTime('hh:nn:ss', duration);
  filesize := RMGetFileSize(RecFileName);
  if filesize > MB then begin
    sFilesize := Format('%.2f', [filesize / MB])+ ' MB';
  end else
    sFilesize := IntToStr(filesize div 1024) + ' KB';
  pFilesize.Caption := sFilesize;
  try
    pBitrate.Caption := IntToStr(Trunc(filesize/(duration*86400)* 8/1024)) + ' Kbps';
  except
  end;
end;

procedure TfmMain.pFilenameClick(Sender: TObject);
begin
  if fileExists(RecFileName) then
    ShellExecute(Self.Handle,'open',Pchar(RecFileName),'','',SW_SHOWNORMAL)
  else begin
    ShowError('The file does not exist: ' + RecFileName);
    FileLabelAsLink(False); 
  end;
end;

procedure TfmMain.btPauseClick(Sender: TObject);
begin
  if btPause.Caption = 'Pause' then begin
    //Changing from recording to pause
    btPause.Caption := 'Resume';
    pStatus.Caption := 'Paused';
    tmr.Enabled := False;
    SetLincoPauseFlag(True);
    PauseStartTime := Now;
  end else begin
    //Changing from pause to recording
    btPause.Caption := 'Pause';
    pStatus.Caption := 'Recording...';
    PauseTotalTime := PauseTotalTime + (Now - PauseStartTime);
    SetLincoPauseFlag(False);
    tmr.Enabled := True;
  end;
end;

procedure TfmMain.FormCreate(Sender: TObject);
var DefaultCmdShell:String;
begin
  ini := TRMIniFile.Create(ChangeFileExt(Application.Exename,'.ini'));
  fmMain.Top := ini.Read('Window','top',300);
  fmMain.Left := ini.Read('Window','left',300);
  fmMain.Caption := PROGNAME;
  Application.Title := PROGNAME;
  if WindowsNT then
    DefaultCmdShell := 'cmd'
  else
    DefaultCmdShell := 'command.com';
  CmdShell := ini.Read(INIGENERAL, 'cmdshell', DefaultCmdShell);
  if not FileExists(PROFILEDIR + '\Default.ini') then begin
    try
      MkDir(PROFILEDIR);
    except
    end;
    MakeEmptyFile(PROFILEDIR + '\Default.ini');
  end;
  UpdateProfileList;
end;

procedure TfmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ini.Write('Window','top',fmMain.Top);
  ini.Write('Window','left',fmMain.Left);
  ini.Write(INIGENERAL, 'cmdshell', CmdShell);
  if nowRecording then begin
    btStop.Click;
    sleep(2000);
  end;  
end;

procedure TfmMain.About1Click(Sender: TObject);
var abi:TAboutBoxInfo;
begin
  abi.ProgramName := PROGNAME;
  abi.Version := PROGVER;
  abi.ReleaseDate := RELDATE;
  abi.Copyright := COPYRIGHT;
  abi.Description := PROGDESC;
  abi.EMail := AUTHMAIL;
  abi.WebPage := WEBSITE;
  abi.Icon := Application.Icon;
  RMShowModalAboutBox(abi);
end;

procedure TfmMain.Configure1Click(Sender: TObject);
begin
  fmConfig := TfmConfig.Create(Self);
  fmConfig.ShowModal;
  UpdateProfileList;
end;

procedure TfmMain.RecordingMixer1Click(Sender: TObject);
begin
  ShellExecute(Self.Handle,PChar('open'),PChar('sndvol32'),PChar('/rec'),'',SW_SHOWNORMAL);
end;

procedure TfmMain.VolumeControl1Click(Sender: TObject);
begin
  ShellExecute(Self.Handle,PChar('open'),PChar('sndvol32'),'','',SW_SHOWNORMAL);
end;

end.
