unit unRM_IniWork;
{
    unRM_IniWork - an utility unit for working with INI-files 
    Copyright (C) 2003 Roman Mamedov
    <E-Mail: roman@rm.pp.ru, Website: http://rm.pp.ru/?en>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
interface

uses IniFiles;

type
  TRMIniFile = class(TIniFile)
    constructor Create(const FileName: string);
    function Read(const Section, Ident, Default: string): string; overload;
    function Read(const Section, Ident: string; Default: Integer): Integer;
      overload;
    function Read(const Section, Ident: string; Default: Boolean): Boolean;
      overload;
    function Read(const Section, Ident: string; Default: Double): Double;
      overload;

    procedure Write(const Section, Ident, Value: string); overload;
    procedure Write(const Section, Ident: string; Value: Integer); overload;
    procedure Write(const Section, Ident: string; Value: Boolean); overload;
    procedure Write(const Section, Ident: string; Value: Double); overload;    

    procedure Rename(const Section, OldIdent, NewIdent, Default: string);
    procedure MoveValue(const Section1, Section2, Ident1, Ident2: string);
    procedure MoveValueToIni(var Ini2:TRMIniFile; const Section1, Ident1, Section2, Ident2: string);
  private

  end;

implementation

procedure TRMIniFile.MoveValue(const Section1, Section2, Ident1, Ident2: string);
begin
  if ValueExists(Section1,Ident1) then begin
    WriteString(Section2, Ident2, ReadString(Section1, Ident1, ''));
    DeleteKey(Section1,Ident1);
  end;
end;

procedure TRMIniFile.MoveValueToIni(var Ini2:TRMIniFile; const Section1, Ident1, Section2, Ident2: string);
begin
  if ValueExists(Section1,Ident1) then begin
    Ini2.WriteString(Section2,Ident2,ReadString(Section1, Ident1, ''));
    DeleteKey(Section1,Ident1);
  end;
end;

procedure TRMIniFile.Rename(const Section, OldIdent, NewIdent, Default: string);
begin
  if not ValueExists(Section, NewIdent) then
    WriteString(Section, NewIdent, ReadString(Section, OldIdent, Default));
  DeleteKey(Section, OldIdent);
end;

constructor TRMIniFile.Create(const FileName: string);
begin
  inherited;
end;

procedure TRMIniFile.Write(const Section, Ident, Value: string);
begin
  WriteString(Section, Ident, Value);
end;

procedure TRMIniFile.Write(const Section, Ident: string; Value: Integer);
begin
  WriteInteger(Section, Ident, Value);
end;

procedure TRMIniFile.Write(const Section, Ident: string; Value: Boolean);
begin
  WriteBool(Section, Ident, Value);
end;

procedure TRMIniFile.Write(const Section, Ident: string; Value: Double);
begin
  WriteFloat(Section, Ident, Value);
end;

function TRMIniFile.Read(const Section, Ident, Default: string): string;
begin
  Result := ReadString(Section, Ident, Default);
end;

function TRMIniFile.Read(const Section, Ident: string; Default: Integer):
  Integer;
begin
  Result := ReadInteger(Section, Ident, Default);
end;

function TRMIniFile.Read(const Section, Ident: string; Default: Boolean):
  Boolean;
begin
  Result := ReadBool(Section, Ident, Default);
end;

function TRMIniFile.Read(const Section, Ident: string; Default: Double):
  Double;
begin
  Result := ReadFloat(Section, Ident, Default);
end;

end.

