unit unrm_free_utils;
{
    unRM_Free_Utils - an utility unit with various useful functions.
    Copyright (C) 2003 Roman Mamedov (roman@rm.pp.ru, http://rm.pp.ru/?en)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
{
        v1.0    02.08.2003
                Started version numbering.
}
interface

uses {$IFDEF win32}Windows,{$ENDIF} SysUtils, Classes;

{$IFDEF win32}
function WindowsNT:Boolean;
function RM_FindWindow(const WinText: string; RequireFullMatch:Boolean): HWnd;
// Returns the handle of a window which caption contains WinText,
// or 0 if not found
{$ENDIF}

function rm_explode(const Separator, Str: string): TStringList;
// Divides Str into 'words' using Separator as boundaries      
function SimpleReplaceChar(const S: string; const Old, Neo: Char):string;

implementation

{$IFDEF win32}
function WindowsNT:Boolean;
var OVI: TOSVERSIONINFO;
begin
  OVI.dwOSVersionInfoSize := SizeOf(OSVERSIONINFO);
  GetVersionEx(OVI);
  Result := OVI.dwPlatformId = VER_PLATFORM_WIN32_NT;
end;
{$ENDIF}

{$IFDEF win32}
function RM_FindWindow(const WinText: string; RequireFullMatch:Boolean): HWnd;
var
  CaptionStr: string;
  Count: Integer;
  WindowText: array[0..255] of Char;
  H: HWnd;
  WindowFound: Boolean;
begin
  Count := 0;
  Result := 0;
  WindowFound := False;
  CaptionStr := AnsiUpperCase(WinText);
  {get handle of first window}
  H := FindWindow(nil, nil);
  repeat
    Inc(Count);
    if GetWindowText(H, @WindowText, SizeOf(WindowText) - 1) <> 0 then begin
      if RequireFullMatch then
        WindowFound := CaptionStr = Uppercase(WindowText)
      else
        WindowFound := Pos(CaptionStr, Uppercase(WindowText))>0;      
    end;
    if not WindowFound then H := GetWindow(H, GW_HWNDNEXT);
  until WindowFound or (H = 0) or (Count > 1000);
  if WindowFound then Result := H;
end;
{$ENDIF}

function SimpleReplaceChar(const S: string; const Old, Neo: Char):string;
var
  C: Integer;
begin
  Result := S;
  if Length(Result) = 0 then Exit;
  for C := 1 to Length(Result) do
    if Result[C] = Old then Result[C] := Neo;
end;


function rm_explode(const Separator, Str: string): TStringList;
var
  S: string;
  P: Integer;
begin
  S := Str;
  Result := TStringList.Create;
  while True do
  begin
    P := Pos(Separator, S);
    if P = 0 then
    begin
      Result.Add(Trim(S));
      Break;
    end else
    begin
      Result.Add(Trim(Copy(S, 0, P - 1)));
      S := Copy(S, P + 1, 10000000);
    end;
  end;
end;

end.
